'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

exports.realGetConfig = realGetConfig;

exports.default = function (env, cwd) {
  const paths = (0, _paths2.default)(cwd);
  const pkg = JSON.parse((0, _fs.readFileSync)(paths.appPackageJson, 'utf-8'));
  return realGetConfig('.beesrc', env, pkg, paths);
};

var _fs = require('fs');

var _stripJsonComments = require('strip-json-comments');

var _stripJsonComments2 = _interopRequireDefault(_stripJsonComments);

var _isPlainObject = require('is-plain-object');

var _isPlainObject2 = _interopRequireDefault(_isPlainObject);

var _parseJsonPretty = require('parse-json-pretty');

var _parseJsonPretty2 = _interopRequireDefault(_parseJsonPretty);

var _paths = require('../config/paths');

var _paths2 = _interopRequireDefault(_paths);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

require('./registerBabel');

function merge(oldObj, newObj) {
  for (const key in newObj) {
    if (Array.isArray(newObj[key]) && Array.isArray(oldObj[key])) {
      oldObj[key] = oldObj[key].concat(newObj[key]);
    } else if ((0, _isPlainObject2.default)(newObj[key]) && (0, _isPlainObject2.default)(oldObj[key])) {
      oldObj[key] = (0, _assign2.default)(oldObj[key], newObj[key]);
    } else {
      oldObj[key] = newObj[key];
    }
  }
}

function getConfig(configFile, paths) {
  const rcConfig = paths.resolveApp(configFile);
  const jsConfig = paths.resolveApp(`${configFile}.js`);

  if ((0, _fs.existsSync)(rcConfig)) {
    if (process.env.NODE_ENV === 'development' && (0, _fs.existsSync)(jsConfig)) {
      console.error(`Config error: You must delete ${rcConfig} if you want to use ${jsConfig}`);
    }
    return (0, _parseJsonPretty2.default)((0, _stripJsonComments2.default)((0, _fs.readFileSync)(rcConfig, 'utf-8')), `./${configFile}`);
  } else if ((0, _fs.existsSync)(jsConfig)) {
    return require(jsConfig); // eslint-disable-line
  } else {
    return {};
  }
}

function replaceNpmVariables(value, pkg) {
  if (typeof value === 'string') {
    return value.replace('$npm_package_name', pkg.name).replace('$npm_package_version', pkg.version);
  } else {
    return value;
  }
}

function realGetConfig(configFile, env, pkg = {}, paths) {
  env = env || 'development';
  const config = getConfig(configFile, paths);
  if (config.env) {
    if (config.env[env]) merge(config, config.env[env]);
    delete config.env;
  }

  return (0, _keys2.default)(config).reduce((memo, key) => {
    memo[key] = replaceNpmVariables(config[key], pkg);
    return memo;
  }, {});
}